@@ -13,5 +13,5 @@ export GOPATH=$project/build
 go install github.com/varlink/go/cmd/varlink-go-interface-generator
 go install gotest.tools/gotestsum
 
-go generate github.com/pascomnet/nomad-driver-podman/iopodman
-go build
\ No newline at end of file
+go generate github.com/hashicorp/nomad-driver-podman/iopodman
+go build
