@@ -22,23 +22,25 @@ import (
 	"fmt"
 	"io/ioutil"
 	"os"
+	"os/exec"
+	"os/user"
+
 	"path/filepath"
 	"reflect"
 	"strconv"
 	"testing"
 	"time"
 
-	"github.com/hashicorp/consul/lib/freeport"
 	"github.com/hashicorp/go-hclog"
+	"github.com/hashicorp/nomad-driver-podman/iopodman"
 	"github.com/hashicorp/nomad/client/taskenv"
-	ctestutil "github.com/hashicorp/nomad/client/testutil"
+	"github.com/hashicorp/nomad/helper/freeport"
 	"github.com/hashicorp/nomad/helper/testlog"
 	"github.com/hashicorp/nomad/helper/uuid"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/plugins/drivers"
 	dtestutil "github.com/hashicorp/nomad/plugins/drivers/testutils"
 	tu "github.com/hashicorp/nomad/testutil"
-	"github.com/pascomnet/nomad-driver-podman/iopodman"
 	"github.com/stretchr/testify/require"
 	"github.com/varlink/go/varlink"
 )
@@ -47,8 +49,23 @@ var (
 	// busyboxLongRunningCmd is a busybox command that runs indefinitely, and
 	// ideally responds to SIGINT/SIGTERM.  Sadly, busybox:1.29.3 /bin/sleep doesn't.
 	busyboxLongRunningCmd = []string{"nc", "-l", "-p", "3000", "127.0.0.1"}
+	varlinkSocketPath     = ""
 )
 
+func init() {
+	user, _ := user.Current()
+	procFilesystems, err := getProcFilesystems()
+
+	if err != nil {
+		fmt.Fprintf(os.Stderr, "error: %v\n", err)
+		os.Exit(1)
+	}
+
+	socketPath := guessSocketPath(user, procFilesystems)
+
+	varlinkSocketPath = socketPath
+}
+
 func createBasicResources() *drivers.Resources {
 	res := drivers.Resources{
 		NomadResources: &structs.AllocatedTaskResources{
@@ -71,9 +88,8 @@ func createBasicResources() *drivers.Resources {
 // A driver plugin interface and cleanup function is returned
 func podmanDriverHarness(t *testing.T, cfg map[string]interface{}) *dtestutil.DriverHarness {
 
-	ctestutil.RequireRoot(t)
-
 	d := NewPodmanDriver(testlog.HCLogger(t)).(*Driver)
+	d.podmanClient = newPodmanClient()
 	d.config.Volumes.Enabled = true
 	if enforce, err := ioutil.ReadFile("/sys/fs/selinux/enforce"); err == nil {
 		if string(enforce) == "1" {
@@ -249,6 +265,7 @@ func TestPodmanDriver_Start_Wait_AllocDir(t *testing.T) {
 
 	exp := []byte{'w', 'i', 'n'}
 	file := "output.txt"
+	allocDir := "/mnt/alloc"
 
 	taskCfg := newTaskConfig("", []string{
 		"sh",
@@ -256,6 +273,7 @@ func TestPodmanDriver_Start_Wait_AllocDir(t *testing.T) {
 		fmt.Sprintf(`echo -n %s > $%s/%s; sleep 1`,
 			string(exp), taskenv.AllocDir, file),
 	})
+	taskCfg.Volumes = []string{fmt.Sprintf("alloc/:%s", allocDir)}
 	task := &drivers.TaskConfig{
 		ID:        uuid.Generate(),
 		Name:      "start_wait_allocDir",
@@ -502,7 +520,8 @@ func TestPodmanDriver_PortMap(t *testing.T) {
 	if !tu.IsCI() {
 		t.Parallel()
 	}
-	ports := freeport.GetT(t, 2)
+	ports := freeport.MustTake(2)
+	defer freeport.Return(ports)
 
 	taskCfg := newTaskConfig("", busyboxLongRunningCmd)
 
@@ -585,10 +604,15 @@ func TestPodmanDriver_Init(t *testing.T) {
 	}
 
 	// only test --init if catatonit is installed
-	_, err := os.Stat("/usr/libexec/podman/catatonit")
+	initPath := "/usr/libexec/podman/catatonit"
+	_, err := os.Stat(initPath)
 	if os.IsNotExist(err) {
-		t.Skip("Skipping --init test because catatonit is not installed")
-		return
+		path, err := exec.LookPath("catatonit")
+		if err != nil {
+			t.Skip("Skipping --init test because catatonit is not installed")
+			return
+		}
+		initPath = path
 	}
 
 	taskCfg := newTaskConfig("", []string{
@@ -598,6 +622,7 @@ func TestPodmanDriver_Init(t *testing.T) {
 	})
 	// enable --init
 	taskCfg.Init = true
+	taskCfg.InitPath = initPath
 
 	task := &drivers.TaskConfig{
 		ID:        uuid.Generate(),
@@ -642,10 +667,22 @@ func TestPodmanDriver_OOM(t *testing.T) {
 		// Incrementally creates a bigger and bigger variable.
 		"sh",
 		"-c",
-		"x=a; while true; do eval x='$x$x'; done",
+		"tail /dev/zero",
 	})
-	// enable --init
+
+	// only enable init if catatonit is installed
+	initPath := "/usr/libexec/podman/catatonit"
+	_, err := os.Stat(initPath)
+	if os.IsNotExist(err) {
+		path, err := exec.LookPath("catatonit")
+		if err != nil {
+			t.Skip("Skipping oom test because catatonit is not installed")
+			return
+		}
+		initPath = path
+	}
 	taskCfg.Init = true
+	taskCfg.InitPath = initPath
 
 	task := &drivers.TaskConfig{
 		ID:        uuid.Generate(),
@@ -661,7 +698,7 @@ func TestPodmanDriver_OOM(t *testing.T) {
 	cleanup := d.MkAllocDir(task, true)
 	defer cleanup()
 
-	_, _, err := d.StartTask(task)
+	_, _, err = d.StartTask(task)
 	require.NoError(t, err)
 
 	defer d.DestroyTask(task.ID, true)
@@ -778,8 +815,14 @@ func TestPodmanDriver_Swap(t *testing.T) {
 	require.Equal(t, int64(52428800), inspectData.HostConfig.Memory)
 	require.Equal(t, int64(41943040), inspectData.HostConfig.MemoryReservation)
 	require.Equal(t, int64(104857600), inspectData.HostConfig.MemorySwap)
-	require.Equal(t, int64(60), inspectData.HostConfig.MemorySwappiness)
 
+	procFilesystems, err := getProcFilesystems()
+	if err == nil {
+		cgroupv2 := isCGroupV2(procFilesystems)
+		if cgroupv2 == false {
+			require.Equal(t, int64(60), inspectData.HostConfig.MemorySwappiness)
+		}
+	}
 }
 
 // check tmpfs mounts
@@ -830,9 +873,9 @@ func TestPodmanDriver_Tmpfs(t *testing.T) {
 
 	// see if tmpfs was propagated to podman
 	inspectData := inspectContainer(t, containerName)
-	expectedFilesystem := map[string]string {
-		"/tmpdata1" : "rw,rprivate,noexec,nosuid,nodev,tmpcopyup",
-		"/tmpdata2" : "rw,rprivate,noexec,nosuid,nodev,tmpcopyup",
+	expectedFilesystem := map[string]string{
+		"/tmpdata1": "rw,rprivate,nosuid,nodev,tmpcopyup",
+		"/tmpdata2": "rw,rprivate,nosuid,nodev,tmpcopyup",
 	}
 	require.Exactly(t, expectedFilesystem, inspectData.HostConfig.Tmpfs)
 
@@ -842,6 +885,107 @@ func TestPodmanDriver_Tmpfs(t *testing.T) {
 	require.Contains(t, tasklog, " tmpfs on /tmpdata2 type tmpfs ")
 }
 
+// check default capabilities
+func TestPodmanDriver_DefaultCaps(t *testing.T) {
+	taskCfg := newTaskConfig("", busyboxLongRunningCmd)
+	inspectData := startDestroyInspect(t, taskCfg, "defaultcaps")
+
+	// a default container should not have SYS_TIME
+	require.NotContains(t, inspectData.EffectiveCaps, "CAP_SYS_TIME")
+	// a default container gets MKNOD cap
+	require.Contains(t, inspectData.EffectiveCaps, "CAP_MKNOD")
+}
+
+// check modified capabilities (CapAdd/CapDrop)
+func TestPodmanDriver_Caps(t *testing.T) {
+	taskCfg := newTaskConfig("", busyboxLongRunningCmd)
+	// 	cap_add = [
+	//     "SYS_TIME",
+	//   ]
+	taskCfg.CapAdd = []string{"SYS_TIME"}
+	// 	cap_drop = [
+	//     "MKNOD",
+	//   ]
+	taskCfg.CapDrop = []string{"MKNOD"}
+
+	inspectData := startDestroyInspect(t, taskCfg, "caps")
+
+	// we added SYS_TIME, so we should see it in inspect
+	require.Contains(t, inspectData.EffectiveCaps, "CAP_SYS_TIME")
+	// we dropped CAP_MKNOD, so we should NOT see it in inspect
+	require.NotContains(t, inspectData.EffectiveCaps, "CAP_MKNOD")
+}
+
+// TestPodmanDriver_NetworkMode asserts we can specify different network modes
+// Default podman cni subnet 10.88.0.0/16
+func TestPodmanDriver_NetworkMode(t *testing.T) {
+	if !tu.IsCI() {
+		t.Parallel()
+	}
+
+	testCases := []struct {
+		mode     string
+		expected string
+		gateway  string
+	}{
+		{
+			mode:    "host",
+			gateway: "",
+		},
+		{
+			// https://github.com/containers/libpod/issues/6618
+			// bridge mode info is not fully populated for inspect command
+			// so we can only make certain assertions
+			mode:    "bridge",
+			gateway: "10.88.0.1",
+		},
+		{
+			// slirp4netns information not populated by podman and
+			// is not supported for root containers
+			// https://github.com/containers/libpod/issues/6097
+			mode: "slirp4netns",
+		},
+		{
+			mode:    "none",
+			gateway: "",
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(fmt.Sprintf("%s_mode_%s", t.Name(), tc.mode), func(t *testing.T) {
+
+			taskCfg := newTaskConfig("", busyboxLongRunningCmd)
+			taskCfg.NetworkMode = tc.mode
+
+			task := &drivers.TaskConfig{
+				ID:        uuid.Generate(),
+				Name:      fmt.Sprintf("network_mode_%s", tc.mode),
+				AllocID:   uuid.Generate(),
+				Resources: createBasicResources(),
+			}
+
+			require.NoError(t, task.EncodeConcreteDriverConfig(&taskCfg))
+
+			d := podmanDriverHarness(t, nil)
+			defer d.MkAllocDir(task, true)()
+
+			containerName := BuildContainerName(task)
+			_, _, err := d.StartTask(task)
+			require.NoError(t, err)
+
+			defer d.DestroyTask(task.ID, true)
+
+			require.NoError(t, d.WaitUntilStarted(task.ID, time.Duration(tu.TestMultiplier()*3)*time.Second))
+
+			inspectData := inspectContainer(t, containerName)
+			if tc.mode == "host" {
+				require.Equal(t, "host", inspectData.HostConfig.NetworkMode)
+			}
+			require.Equal(t, tc.gateway, inspectData.NetworkSettings.Gateway)
+		})
+	}
+}
+
 // read a tasks logfile into a string, fail on error
 func readLogfile(t *testing.T, task *drivers.TaskConfig) string {
 	logfile := filepath.Join(filepath.Dir(task.StdoutPath), fmt.Sprintf("%s.stdout.0", task.Name))
@@ -852,9 +996,7 @@ func readLogfile(t *testing.T, task *drivers.TaskConfig) string {
 }
 
 func newTaskConfig(variant string, command []string) TaskConfig {
-	// busyboxImageID is the ID stored in busybox.tar
-	busyboxImageID := "docker://busybox"
-	// busyboxImageID := "busybox:1.29.3"
+	busyboxImageID := "docker://docker.io/library/busybox:latest"
 
 	image := busyboxImageID
 
@@ -896,8 +1038,7 @@ func getContainer(t *testing.T, containerName string) iopodman.Container {
 }
 
 func getPodmanConnection(ctx context.Context) (*varlink.Connection, error) {
-	// FIXME: a parameter for the socket would be nice
-	varlinkConnection, err := varlink.NewConnection(ctx, "unix://run/podman/io.podman")
+	varlinkConnection, err := varlink.NewConnection(ctx, varlinkSocketPath)
 	return varlinkConnection, err
 }
 
@@ -907,8 +1048,9 @@ func newPodmanClient() *PodmanClient {
 		Level: hclog.LevelFromString("DEBUG"),
 	})
 	client := &PodmanClient{
-		ctx:    context.Background(),
-		logger: testLogger,
+		ctx:               context.Background(),
+		logger:            testLogger,
+		varlinkSocketPath: varlinkSocketPath,
 	}
 	return client
 }
@@ -918,3 +1060,42 @@ func getPodmanDriver(t *testing.T, harness *dtestutil.DriverHarness) *Driver {
 	require.True(t, ok)
 	return driver
 }
+
+// helper to start, destroy and inspect a long running container
+func startDestroyInspect(t *testing.T, taskCfg TaskConfig, taskName string) iopodman.InspectContainerData {
+	if !tu.IsCI() {
+		t.Parallel()
+	}
+
+	task := &drivers.TaskConfig{
+		ID:        uuid.Generate(),
+		Name:      taskName,
+		AllocID:   uuid.Generate(),
+		Resources: createBasicResources(),
+	}
+	require.NoError(t, task.EncodeConcreteDriverConfig(&taskCfg))
+
+	d := podmanDriverHarness(t, nil)
+	cleanup := d.MkAllocDir(task, true)
+	defer cleanup()
+
+	containerName := BuildContainerName(task)
+
+	_, _, err := d.StartTask(task)
+	require.NoError(t, err)
+
+	defer d.DestroyTask(task.ID, true)
+
+	// Attempt to wait
+	waitCh, err := d.WaitTask(context.Background(), task.ID)
+	require.NoError(t, err)
+
+	select {
+	case <-waitCh:
+		t.Fatalf("wait channel should not have received an exit result")
+	case <-time.After(time.Duration(tu.TestMultiplier()*2) * time.Second):
+	}
+	// inspect container to learn about the actual podman limits
+	inspectData := inspectContainer(t, containerName)
+	return inspectData
+}
